'euse strict'

const xtend = require('xtend')

function addServerMethod (method) {
  if (Array.isArray(method)) {
    for (var i = 0; i < method.length; i++) {
      _addServerMethod(this, method[i])
    }
  } else {
    _addServerMethod(this, method)
  }
}

function _addServerMethod (instance, method) {
  const methodName = Object.keys(method)[0]

  if (hasMethod(instance, methodName)) {
    throw new Error(`method '${methodName}' already loaded.`)
  }
  instance[methodName] = method[methodName]
}

function hasMethod (instance, method) {
  if (method === undefined) {
    method = instance
    instance = this
  }
  if (typeof method !== 'string') {
    throw new Error('method name must be a string')
  }
  return !!instance[method]
}

function addRRMethod (RR) {
  return function _addRRMethodClosure (method) {
    if (Array.isArray(method)) {
      for (var i = 0; i < method.length; i++) {
        _addRRMethod(RR, method[i])
      }
    } else {
      _addRRMethod(RR, method)
    }
  }
}

function _addRRMethod (RR, method) {
  const methodName = Object.keys(method)[0]

  if (hasMethod(RR, Object.keys(method)[0])) {
    throw new Error(`Request || Reply method '${methodName}' already present.`)
  }

  xtend(RR.prototype, method)
}

function addOnRequestMethod (onRequesteMethods) {
  return function _addOnRequestMethod (method) {
    onRequesteMethods.push(method)
  }
}

module.exports = {
  addServerMethod: addServerMethod,
  hasMethod: hasMethod,
  addRRMethod: addRRMethod,
  addOnRequestMethod: addOnRequestMethod
}
